package ru.dbotthepony.mc.otm.datagen

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.block.state.properties.BlockStateProperties
import net.minecraftforge.client.model.generators.ConfiguredModel
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.block.RotatableMatteryBlock
import ru.dbotthepony.mc.otm.core.get
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.blocks.MatteryBlockStateProvider
import ru.dbotthepony.mc.otm.datagen.items.MatteryItemModelProvider
import ru.dbotthepony.mc.otm.datagen.models.MatteryBlockModelProvider
import ru.dbotthepony.mc.otm.datagen.models.MatteryModelBuilder
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.registry.MRegistry

fun addDecorativeData(blockStateProvider: MatteryBlockStateProvider, itemModelProvider: MatteryItemModelProvider, blockModelProvider: MatteryBlockModelProvider) {
	for ((color, block) in MRegistry.DECORATIVE_CRATE.allBlocks) {
		if (color == null) {
			blockModelProvider.decorativeCubeAll(block, "crates/crate_colorless_borderless")
		} else {
			blockModelProvider.decorativeCubeAll(block, "crates/crate_borderless_${color.name.lowercase()}")
		}

		blockStateProvider.simpleBlockM(block)
	}

	DataGen.decorativeCubeAll(MBlocks.CARBON_FIBRE_BLOCK)

	DataGen.decoratives(MRegistry.TRITANIUM_BLOCK)
	DataGen.decoratives(MRegistry.FLOOR_TILES)
	DataGen.decoratives(MRegistry.UNREFINED_FLOOR_TILES)
	DataGen.decoratives(MRegistry.VENT)
	DataGen.decoratives(MRegistry.VENT_ALTERNATIVE)

	blockModelProvider.decorativeGlassAll(MRegistry.INDUSTRIAL_GLASS.allBlocks.values)
	blockStateProvider.simpleBlockM(MRegistry.INDUSTRIAL_GLASS.allBlocks.values)

	for ((block, colors) in MRegistry.TRITANIUM_STRIPED_BLOCK.blocksWithColor) {
		DataGen.decorativeColumn(
			block,
			"stripe/${block.registryName!!.path}",
			"tritanium_block_${colors.first.name.lowercase()}"
		)
	}

	DataGen.decorativeCubeAll(MBlocks.DANGER_STRIPE_BLOCK)
	DataGen.decorativeColumn(MBlocks.METAL_BEAM, "metal_beam_side", "metal_beam_top")

	var labLampOn: MatteryModelBuilder? = null
	var labLampOff: MatteryModelBuilder? = null

	blockModelProvider.exec {
		val top = ResourceLocation(DataGen.MOD_ID, "block/decorative/metal_beam_top")

		labLampOn = it.cube(
			MBlocks.LABORATORY_LAMP.registryName!!.path,
			top,
			top,
			ResourceLocation(DataGen.MOD_ID, "block/decorative/laboratory_lamp_front"),
			top,
			top,
			top,
		).texture("particle", top)

		labLampOff = it.cube(
			MBlocks.LABORATORY_LAMP.registryName!!.path + "_unlit",
			top,
			top,
			ResourceLocation(DataGen.MOD_ID, "block/decorative/laboratory_lamp_front_off"),
			top,
			top,
			top,
		).texture("particle", top)
	}

	itemModelProvider.block(MItems.LABORATORY_LAMP)
	itemModelProvider.block(MItems.LABORATORY_LAMP_INVERTED, MItems.LABORATORY_LAMP.registryName!!.path + "_unlit")
	itemModelProvider.block(MItems.DANGER_STRIPE_BLOCK)
	itemModelProvider.block(MItems.METAL_BEAM)

	blockStateProvider.exec {
		blockStateProvider.getVariantBuilder(MBlocks.LABORATORY_LAMP).forAllStates {
			return@forAllStates ConfiguredModel.builder()
				.modelFile(if (it[BlockStateProperties.LIT]) labLampOn!! else labLampOff!!)
				.rotationX(it[RotatableMatteryBlock.FACING_FULL].toXRotBlockstate())
				.rotationY(it[RotatableMatteryBlock.FACING_FULL].toYRotBlockstate())
				.build()
		}

		blockStateProvider.getVariantBuilder(MBlocks.LABORATORY_LAMP_INVERTED).forAllStates {
			return@forAllStates ConfiguredModel.builder()
				.modelFile(if (it[BlockStateProperties.LIT]) labLampOn!! else labLampOff!!)
				.rotationX(it[RotatableMatteryBlock.FACING_FULL].toXRotBlockstate())
				.rotationY(it[RotatableMatteryBlock.FACING_FULL].toYRotBlockstate())
				.build()
		}
	}

	blockModelProvider.exec {
		for (crate in MRegistry.CARGO_CRATES.blocks.values) {
			it.withExistingParent("${crate.registryName!!.path}_closed", ResourceLocation(OverdriveThatMatters.MOD_ID, "${MNames.CARGO_CRATE}_closed"))
				.texture("texture", "block/cargo_crates/${crate.registryName!!.path}")
				.texture("particle", "block/cargo_crates/${crate.registryName!!.path}")

			it.withExistingParent("${crate.registryName!!.path}_open", ResourceLocation(OverdriveThatMatters.MOD_ID, "${MNames.CARGO_CRATE}_open"))
				.texture("texture", "block/cargo_crates/${crate.registryName!!.path}")
				.texture("particle", "block/cargo_crates/${crate.registryName!!.path}")
		}
	}

	DataGen.decorativeColumn(MBlocks.TRITANIUM_STRIPED_BLOCK, "tritanium_striped_block", "tritanium_block")

	for ((color, glass) in MRegistry.INDUSTRIAL_GLASS_PANE.allBlocks) {
		val name = MRegistry.INDUSTRIAL_GLASS.allBlocks[color]!!.registryName!!.path
		val textureSide = ResourceLocation(DataGen.MOD_ID, "block/decorative/$name")
		val textureRailing = ResourceLocation(DataGen.MOD_ID, "block/decorative/industrial_glass_frame")
		DataGen.pane(glass, textureSide, textureRailing)
	}
}
