package ru.dbotthepony.mc.otm.datagen.recipes

import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.tags.ItemTags
import net.minecraft.world.item.Items
import net.minecraft.world.item.crafting.Ingredient
import net.minecraftforge.common.Tags
import net.minecraftforge.common.Tags.Items.COBBLESTONE
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import ru.dbotthepony.mc.otm.core.registryName
import java.util.function.Consumer

fun addCraftingTableRecipes(consumer: Consumer<FinishedRecipe>) {
	for ((dye, crate) in MRegistry.CARGO_CRATES.blocks) {
		ShapedRecipeBuilder(crate, 1)
			.define('P', MItemTags.TRITANIUM_PLATES)
			.define('C', Tags.Items.CHESTS)
			.define('D', dye.tag)
			.pattern(" D ").pattern("PCP")
			// .unlockedBy("has_dye", has(dye.tag))
			.unlockedBy("has_tritanium_plate", has(MItemTags.TRITANIUM_PLATES))
			.unlockedBy("has_chest", has(Tags.Items.CHESTS))
			.save(consumer)

		ShapelessRecipeBuilder(crate, 1)
			.requires(Ingredient.of(MItemTags.TRITANIUM_CRATES), 1)
			.requires(Ingredient.of(dye.tag), 1)
			.unlockedBy("has_tritanium_crate", has(MItemTags.TRITANIUM_CRATES))
			.save(consumer, "${crate.registryName}_alt")
	}

	ShapedRecipeBuilder(MRegistry.CARGO_CRATES.item, 1)
		.define('P', MItemTags.TRITANIUM_PLATES)
		.define('C', Tags.Items.CHESTS)
		.pattern("PCP")
		.unlockedBy("has_tritanium_plate", has(MItemTags.TRITANIUM_PLATES))
		.unlockedBy("has_chest", has(Tags.Items.CHESTS))
		.save(consumer)

	MatteryRecipe(MBlocks.PLATE_PRESS)
		.row(MItems.ELECTRIC_PARTS, MItems.ENERGY_BUS, MItems.ELECTRIC_PARTS)
		.row(MItemTags.TRITANIUM_INGOTS, Items.BLAST_FURNACE, MItemTags.TRITANIUM_INGOTS)
		.row(MItemTags.PISTONS, MItemTags.TRITANIUM_INGOTS, MItemTags.PISTONS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItems.ELECTRIC_PARTS)
		.build(consumer)

	MatteryRecipe(MBlocks.PLATE_PRESS)
		.rowB(MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.rowAC(MItemTags.PISTONS, MItemTags.PISTONS)
		.unlockedBy(MItemTags.TRITANIUM_INGOTS)
		.unlockedBy(MItems.ELECTRIC_PARTS)
		.build(consumer, "advanced")

	MatteryRecipe(MBlocks.TRITANIUM_STRIPED_BLOCK, 24)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, COBBLESTONE, MItemTags.TRITANIUM_PLATES)
		.rowAB(Tags.Items.DYES_YELLOW, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(Tags.Items.DYES_YELLOW)
		.build(consumer)

	MatteryRecipe(MRegistry.TRITANIUM_BLOCK.item, 24)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, COBBLESTONE, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// Машины
	MatteryRecipe(MItems.MATTER_RECYCLER)
		.row(MItems.MATTER_CAPACITOR_PARTS, Items.HOPPER, MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItems.MATTER_CABLE, MItems.MATTER_IO_PORT, MItems.MATTER_CABLE)
		.unlockedBy(MItems.MATTER_CABLE)
		.build(consumer)

	// Блоки
	MatteryRecipe(MItems.MATTER_CAPACITOR_BANK)
		.row(Tags.Items.GLASS, MItemTags.IRON_PLATES, Tags.Items.GLASS)
		.row(MItemTags.IRON_PLATES, MItems.MACHINE_FRAME, MItemTags.IRON_PLATES)
		.row(MItems.MATTER_CABLE, MItems.MATTER_IO_PORT, MItems.MATTER_CABLE)
		.unlockedBy(MItems.MATTER_CABLE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_BANK)
		.row(Tags.Items.GLASS, MItemTags.IRON_PLATES, Tags.Items.GLASS)
		.row(MItemTags.IRON_PLATES, MItems.MACHINE_FRAME, MItemTags.IRON_PLATES)
		.row(MItems.ELECTRIC_PARTS, MItems.ENERGY_BUS, MItems.ELECTRIC_PARTS)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_STABILIZER)
		.row(MItemTags.ADVANCED_CIRCUIT, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.ADVANCED_CIRCUIT)
		.row(MItems.MATTER_TRANSFORM_MATRIX, MItems.MACHINE_FRAME, MItems.MATTER_TRANSFORM_MATRIX)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_LIMITER, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_LIMITER)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.PORTABLE_GRAVITATION_STABILIZER)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.GRAVITATION_FIELD_LIMITER, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_LIMITER)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_FIELD_SENSOR)
		.rowB(MItemTags.BASIC_CIRCUIT)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTROMAGNET, MItemTags.TRITANIUM_PLATES)
		.rowB(MItemTags.IRON_PLATES)
		.unlockedBy(MItems.ELECTROMAGNET)
		.build(consumer)

	MatteryRecipe(MItems.GRAVITATION_FIELD_LIMITER)
		.row(Tags.Items.ENDER_PEARLS, MItemTags.ADVANCED_CIRCUIT, Tags.Items.ENDER_PEARLS)
		.row(MItemTags.GOLD_WIRES, MItems.QUANTUM_TRANSCEIVER, MItemTags.GOLD_WIRES)
		.rowB(MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.QUANTUM_TRANSCEIVER)
		.build(consumer)

	MatteryRecipe(MItems.BLACK_HOLE_SCANNER)
		.row(MItemTags.IRON_PLATES, Tags.Items.GLASS_PANES_COLORLESS, MItemTags.IRON_PLATES)
		.row(MItemTags.GOLD_WIRES, MItems.GRAVITATION_FIELD_SENSOR, MItemTags.ADVANCED_CIRCUIT)
		.rowAC(Tags.Items.DUSTS_GLOWSTONE, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.GRAVITATION_FIELD_SENSOR)
		.build(consumer)

	MatteryRecipe(MItems.PHANTOM_ATTRACTOR)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.GLASS_COLORLESS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.TRITANIUM_PLATES, MItems.QUANTUM_TRANSCEIVER, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, ItemTags.BEDS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.QUANTUM_TRANSCEIVER)
		.build(consumer)

	MatteryRecipe(MItems.QUANTUM_TRANSCEIVER, 2)
		.rowAC(MItemTags.COPPER_WIRES, MItemTags.COPPER_WIRES)
		.row(MItemTags.GOLD_WIRES, Tags.Items.ENDER_PEARLS, MItemTags.GOLD_WIRES)
		.row(MItemTags.GOLD_WIRES, MItems.ELECTROMAGNET, MItemTags.GOLD_WIRES)
		.unlockedBy(Tags.Items.ENDER_PEARLS)
		.build(consumer)

	MatteryRecipe(MItems.ELECTROMAGNET)
		.row(MItemTags.COPPER_WIRES, Tags.Items.INGOTS_IRON, MItemTags.COPPER_WIRES)
		.unlockedBy(Tags.Items.ENDER_PEARLS)
		.build(consumer)

	MatteryRecipe(MItems.ENERGY_SERVO)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ENERGY_BUS, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItems.ENERGY_BUS)
		.build(consumer)

	MatteryRecipe(MItems.MIRROR_COMPOUND, 3)
		.row(Tags.Items.GLASS_PANES_COLORLESS, Tags.Items.GLASS_PANES_COLORLESS, Tags.Items.GLASS_PANES_COLORLESS)
		.row(MItemTags.IRON_PLATES, MItemTags.IRON_PLATES, MItemTags.IRON_PLATES)
		.unlockedBy(MItemTags.IRON_PLATES)
		.unlockedBy(Tags.Items.GLASS_PANES_COLORLESS)
		.build(consumer)

	// броня
	MatteryRecipe(MItems.TRITANIUM_HELMET)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_PANTS)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.rowAC(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_CHESTPLATE)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.TRITANIUM_BOOTS)
		.rowAC(MItemTags.TRITANIUM_PLATES, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItemTags.IRON_PLATES, MItemTags.TRITANIUM_PLATES)
		.unlockedBy(MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// простые батарейки
	MatteryRecipe(MItems.BATTERY_CRUDE)
		.rowB(Tags.Items.DUSTS_REDSTONE)
		.rowB(Tags.Items.CROPS_POTATO)
		.rowB(Tags.Items.INGOTS_IRON)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_BASIC)
		.rowAC(Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE)
		.rowB(MItems.ELECTRIC_PARTS)
		.rowB(MItemTags.IRON_PLATES)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_NORMAL)
		.rowB(MItems.ELECTRIC_PARTS)
		.row(MItemTags.COPPER_WIRES, MItemTags.IRON_PLATES, MItemTags.COPPER_WIRES)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_DENSE)
		.row(Tags.Items.DUSTS_REDSTONE, MItems.ENERGY_BUS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.GOLD_WIRES, MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.row(Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE, Tags.Items.DUSTS_REDSTONE)
		.build(consumer)

	MatteryRecipe(MItems.BATTERY_CAPACITOR)
		.row(Tags.Items.DUSTS_REDSTONE, MItems.ENERGY_BUS, Tags.Items.DUSTS_REDSTONE)
		.row(MItemTags.GOLD_WIRES, MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.row(MItemTags.GOLD_WIRES, Tags.Items.DUSTS_REDSTONE, MItemTags.GOLD_WIRES)
		.build(consumer)

	// накопители материи
	MatteryRecipe(MItems.MATTER_CAPACITOR_DENSE)
		.row(MItems.MATTER_CAPACITOR_PARTS, Tags.Items.GLASS, MItems.MATTER_CAPACITOR_PARTS)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.ENDER_PEARLS, MItemTags.TRITANIUM_PLATES)
		.rowAC(Tags.Items.GEMS_DIAMOND, Tags.Items.GEMS_DIAMOND)
		.build(consumer)

	// станция андроида
	MatteryRecipe(MItems.ANDROID_STATION)
		.row(MItems.ELECTRIC_PARTS, MItemTags.ADVANCED_CIRCUIT, MItems.ELECTRIC_PARTS)
		.row(MItemTags.TRITANIUM_PLATES, MItems.MACHINE_FRAME, MItemTags.TRITANIUM_PLATES)
		.row(MItemTags.TRITANIUM_PLATES, MItems.ELECTRIC_PARTS, MItemTags.TRITANIUM_PLATES)
		.build(consumer)

	// Энерго меч
	MatteryRecipe(MItems.ENERGY_SWORD)
		.rowBC(MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.rowBC(MItemTags.TRITANIUM_PLATES, MItemTags.GOLD_WIRES)
		.row(MItems.BATTERY_CAPACITOR, MItems.TRITANIUM_SWORD, MItemTags.ADVANCED_CIRCUIT)
		.unlockedBy(MItems.BATTERY_CAPACITOR)
		.buildEnergetic(consumer)

	MatteryRecipe(MItems.LABORATORY_LAMP)
		.row(MItemTags.IRON_PLATES, MItemTags.HARDENED_GLASS_PANES_COLORLESS, MItemTags.IRON_PLATES)
		.row(MItems.MIRROR, Items.GLOWSTONE, MItems.MIRROR)
		.row(MItemTags.TRITANIUM_PLATES, Tags.Items.DUSTS_REDSTONE, MItemTags.TRITANIUM_PLATES)
		.build(consumer)
}
