package ru.dbotthepony.mc.otm.datagen.tags

import net.minecraft.world.effect.MobEffects
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.Items
import net.minecraft.world.item.Tiers
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import ru.dbotthepony.mc.otm.registry.MSoundEvents

fun addTags(tagsProvider: TagsProvider) {
	tagsProvider.items.forge("pistons").add(Items.PISTON)
	tagsProvider.circuits.add("basic", MItems.BASIC_CONTROL_CIRCUIT)
	tagsProvider.circuits.add("advanced", MItems.ADVANCED_CONTROL_CIRCUIT)

	tagsProvider.ingots.add("tritanium", MItems.TRITANIUM_INGOT)

	tagsProvider.plates.add("tritanium", MItems.TRITANIUM_PLATE)
	tagsProvider.plates.add("iron", MItems.IRON_PLATE)
	tagsProvider.plates.add("gold", MItems.GOLD_PLATE)

	tagsProvider.ore("tritanium", MBlocks.TRITANIUM_ORE)
	tagsProvider.ore("tritanium", MBlocks.DEEPSLATE_TRITANIUM_ORE)
	tagsProvider.clump("tritanium", MItems.TRITANIUM_ORE_CLUMP, MBlocks.TRITANIUM_RAW_BLOCK)

	tagsProvider.wires.add("copper", MItems.COPPER_WIRING)
	tagsProvider.wires.add("gold", MItems.GOLD_WIRING)

	tagsProvider.requiresPickaxe(
		MBlocks.MATTER_CABLE,
		MBlocks.STORAGE_CABLE
	)

	tagsProvider.items.forge("hardened_glass").add(MRegistry.INDUSTRIAL_GLASS.allItems.values)
	tagsProvider.items.forge("hardened_glass/colorless").add(MRegistry.INDUSTRIAL_GLASS.item)
	tagsProvider.items.forge("hardened_glass/tinted").add(MRegistry.INDUSTRIAL_GLASS.items.values)

	tagsProvider.items.forge("hardened_glass_panes").add(MRegistry.INDUSTRIAL_GLASS_PANE.allItems.values)
	tagsProvider.items.forge("hardened_glass_panes/colorless").add(MRegistry.INDUSTRIAL_GLASS_PANE.item)
	tagsProvider.items.forge("hardened_glass_panes/tinted").add(MRegistry.INDUSTRIAL_GLASS_PANE.items.values)

	MRegistry.INDUSTRIAL_GLASS.forEachItem { s, _, item -> tagsProvider.items.forge("hardened_glass/$s").add(item) }
	MRegistry.INDUSTRIAL_GLASS_PANE.forEachItem { s, _, item -> tagsProvider.items.forge("hardened_glass_panes/$s").add(item) }

	tagsProvider.blocks.forge("hardened_glass").add(MRegistry.INDUSTRIAL_GLASS.allBlocks.values)
	tagsProvider.blocks.forge("hardened_glass/colorless").add(MRegistry.INDUSTRIAL_GLASS.block)
	tagsProvider.blocks.forge("hardened_glass/tinted").add(MRegistry.INDUSTRIAL_GLASS.blocks.values)

	tagsProvider.blocks.forge("hardened_glass_panes").add(MRegistry.INDUSTRIAL_GLASS_PANE.allBlocks.values)
	tagsProvider.blocks.forge("hardened_glass_panes/colorless").add(MRegistry.INDUSTRIAL_GLASS_PANE.block)
	tagsProvider.blocks.forge("hardened_glass_panes/tinted").add(MRegistry.INDUSTRIAL_GLASS_PANE.blocks.values)

	MRegistry.INDUSTRIAL_GLASS.forEachBlock { s, _, block -> tagsProvider.blocks.forge("hardened_glass/$s").add(block) }
	MRegistry.INDUSTRIAL_GLASS_PANE.forEachBlock { s, _, block -> tagsProvider.blocks.forge("hardened_glass_panes/$s").add(block) }

	tagsProvider.requiresPickaxe(listOf(
		MBlocks.ANDROID_STATION,
		MBlocks.BATTERY_BANK,
		MBlocks.MATTER_DECOMPOSER,
		MBlocks.MATTER_CAPACITOR_BANK,
		MBlocks.PATTERN_STORAGE,
		MBlocks.MATTER_SCANNER,
		MBlocks.MATTER_PANEL,
		MBlocks.MATTER_REPLICATOR,
		MBlocks.MATTER_BOTTLER,
		MBlocks.ENERGY_COUNTER,
		MBlocks.CHEMICAL_GENERATOR,
		MBlocks.PLATE_PRESS,
		MBlocks.MATTER_RECYCLER,

		MBlocks.STORAGE_BUS,
		MBlocks.STORAGE_IMPORTER,
		MBlocks.STORAGE_EXPORTER,

		MBlocks.DRIVE_VIEWER,
		MBlocks.DRIVE_RACK,
		MBlocks.ITEM_MONITOR,
		MBlocks.STORAGE_POWER_SUPPLIER,

		MBlocks.PHANTOM_ATTRACTOR,
		MBlocks.ENERGY_SERVO,
	), Tiers.IRON)

	tagsProvider.requiresPickaxe(MBlocks.TRITANIUM_ORE, Tiers.IRON)
	tagsProvider.requiresPickaxe(MBlocks.DEEPSLATE_TRITANIUM_ORE, Tiers.IRON)
	tagsProvider.requiresPickaxe(MBlocks.TRITANIUM_RAW_BLOCK, Tiers.IRON)

	tagsProvider.requiresPickaxe(MBlocks.TRITANIUM_STRIPED_BLOCK, Tiers.IRON)
	tagsProvider.requiresPickaxe(MBlocks.CARBON_FIBRE_BLOCK, Tiers.IRON)

	tagsProvider.requiresPickaxe(MRegistry.CARGO_CRATES.allBlocks.values, Tiers.IRON)

	tagsProvider.requiresPickaxe(MRegistry.VENT.allBlocks.values, Tiers.IRON)
	tagsProvider.requiresPickaxe(MRegistry.VENT_ALTERNATIVE.allBlocks.values, Tiers.IRON)
	tagsProvider.requiresPickaxe(MRegistry.TRITANIUM_BLOCK.allBlocks.values, Tiers.IRON)
	tagsProvider.requiresPickaxe(MRegistry.TRITANIUM_STRIPED_BLOCK.flatBlocks, Tiers.IRON)

	tagsProvider.requiresPickaxe(MRegistry.FLOOR_TILES.blocks.values)
	tagsProvider.requiresShovel(MRegistry.UNREFINED_FLOOR_TILES.blocks.values)

	tagsProvider.requiresPickaxe(MRegistry.INDUSTRIAL_GLASS.allBlocks.values, Tiers.STONE)
	tagsProvider.requiresPickaxe(MRegistry.INDUSTRIAL_GLASS_PANE.allBlocks.values, Tiers.STONE)
	tagsProvider.requiresPickaxe(MRegistry.DECORATIVE_CRATE.allBlocks.values, Tiers.STONE)

	tagsProvider.requiresPickaxe(MBlocks.LABORATORY_LAMP, Tiers.IRON)
	tagsProvider.requiresPickaxe(MBlocks.LABORATORY_LAMP_INVERTED, Tiers.IRON)
	tagsProvider.requiresPickaxe(MBlocks.METAL_BEAM, Tiers.IRON)
	tagsProvider.requiresPickaxe(MBlocks.DANGER_STRIPE_BLOCK)

	tagsProvider.requiresPickaxe(listOf(
		MBlocks.GRAVITATION_STABILIZER,
		MBlocks.GRAVITATION_STABILIZER_LENS,
	), Tiers.DIAMOND)

	tagsProvider.dragonImmune.add(
		MBlocks.BLACK_HOLE,
		MBlocks.GRAVITATION_STABILIZER,
		MBlocks.GRAVITATION_STABILIZER_LENS,
	)

	tagsProvider.witherImmune.add(
		MBlocks.BLACK_HOLE,
	)

	tagsProvider.androidImmuneEffects.add(
		MobEffects.CONDUIT_POWER,
		MobEffects.HEAL,
		// maybe it makes things go haywire idk
		// MobEffects.HARM,
		MobEffects.REGENERATION,
		MobEffects.WATER_BREATHING,
		MobEffects.POISON,
		// even skeletons can be withered
		// MobEffects.WITHER,
		MobEffects.HEALTH_BOOST,
		MobEffects.ABSORPTION,
		MobEffects.SATURATION,
		MobEffects.DOLPHINS_GRACE,
	)
}
