package ru.dbotthepony.mc.otm.android.feature

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.ChatFormatting
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.level.block.Block
import net.minecraft.world.phys.AABB
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.ServerConfig
import ru.dbotthepony.mc.otm.android.AndroidResearchManager
import ru.dbotthepony.mc.otm.android.AndroidSwitchableFeature
import ru.dbotthepony.mc.otm.capability.MatteryPlayerCapability
import ru.dbotthepony.mc.otm.capability.extractEnergyInnerExact
import ru.dbotthepony.mc.otm.client.render.ResearchIcons
import ru.dbotthepony.mc.otm.core.TextComponent
import ru.dbotthepony.mc.otm.core.Vector
import ru.dbotthepony.mc.otm.core.formatPower
import ru.dbotthepony.mc.otm.core.formatSi
import ru.dbotthepony.mc.otm.core.getEllipsoidBlockPositions
import ru.dbotthepony.mc.otm.core.getEntitiesInEllipsoid
import ru.dbotthepony.mc.otm.core.getExplosionResistance
import ru.dbotthepony.mc.otm.core.minus
import ru.dbotthepony.mc.otm.core.plus
import ru.dbotthepony.mc.otm.core.position
import ru.dbotthepony.mc.otm.core.roundToIntVector
import ru.dbotthepony.mc.otm.core.times
import ru.dbotthepony.mc.otm.network.MatteryPlayerNetworkChannel
import ru.dbotthepony.mc.otm.network.TriggerShockwavePacket
import ru.dbotthepony.mc.otm.registry.AndroidFeatures
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.registry.ShockwaveDamageSource
import kotlin.math.pow
import kotlin.math.roundToInt

class ShockwaveFeature(capability: MatteryPlayerCapability) : AndroidSwitchableFeature(AndroidFeatures.SHOCKWAVE, capability) {
	var cooldown by synchronizer.int()

	private var wasMidair = false
	private var highestSpeed = 0.0
	var airTicks = 0
		private set

	override fun tickClient() {
		if (isActive && ply.isSteppingCarefully && cooldown <= 0 && android.androidEnergy.extractEnergyInnerExact(ServerConfig.Shockwave.ENERGY_COST, true).isPositive && ply.deltaMovement.y < -0.01) {
			ply.deltaMovement += Vector(0.0, -ServerConfig.Shockwave.ACCELERATION / 20.0, 0.0)
		}

		ticker(true)
	}

	fun shockwave() {
		if (cooldown > 0 || !android.androidEnergy.extractEnergyInnerExact(ServerConfig.Shockwave.ENERGY_COST, false).isPositive) {
			return
		}

		cooldown = ServerConfig.Shockwave.COOLDOWN

		// TODO: raycasting
		val entities = ply.level.getEntitiesInEllipsoid(
			ply.position,
			Vector(ServerConfig.Shockwave.RADIUS_HORIZONTAL, ServerConfig.Shockwave.RADIUS_VERTICAL, ServerConfig.Shockwave.RADIUS_HORIZONTAL),
			except = ply,
		) { (it !is LivingEntity || !it.isSpectator && it.isAlive) }

		for ((entity, distanceMultiplier) in entities) {
			val multiplier = (1.0 - distanceMultiplier).pow(1.25)

			// don't hurt items, arrows, etc etc
			if (entity is LivingEntity) {
				entity.hurt(ShockwaveDamageSource(ply), multiplier.toFloat() * ServerConfig.Shockwave.DAMAGE.toFloat())
				entity.deltaMovement += (entity.position - ply.position).normalize() * (multiplier * 3.0)
			} else {
				entity.deltaMovement += (entity.position - ply.position).normalize() * (multiplier * 6.0)
			}
		}

		if (ServerConfig.Shockwave.BREAK_BLOCKS) {
			val rounded = ply.position.roundToIntVector()

			for (blockPos in getEllipsoidBlockPositions(ServerConfig.Shockwave.RADIUS_HORIZONTAL.roundToInt(), ServerConfig.Shockwave.RADIUS_VERTICAL.roundToInt(), ServerConfig.Shockwave.RADIUS_HORIZONTAL.roundToInt())) {
				val newBlockPos = blockPos + rounded

				val blockState = ply.level.getBlockState(newBlockPos)

				if (!blockState.isAir && blockState.getExplosionResistance(ply.level, newBlockPos) <= 0f && ply.level.getBlockEntity(newBlockPos) == null) {
					// Block.dropResources(blockState, ply.level, newBlockPos)
					// blockState.block.destroy(ply.level, newBlockPos, blockState)
					// ply.level.setBlock(newBlockPos, blockState.fluidState.createLegacyBlock(), Block.UPDATE_ALL)

					ply.level.destroyBlock(newBlockPos, true)
				}
			}
		}
	}

	private fun ticker(isClient: Boolean) {
		if (!ply.isOnGround) {
			airTicks = (airTicks + 1).coerceAtMost(3)
		} else {
			airTicks = (airTicks - 1).coerceAtLeast(0)
		}

		if (isActive && cooldown <= 0 && android.androidEnergy.extractEnergyInnerExact(ServerConfig.Shockwave.ENERGY_COST, true).isPositive) {
			val old = wasMidair
			wasMidair = !ply.isOnGround

			if (wasMidair) {
				highestSpeed = (-ply.deltaMovement.y).coerceAtLeast(highestSpeed)
			}

			if (old != wasMidair && !wasMidair && ServerConfig.Shockwave.TERMINAL_VELOCITY <= (highestSpeed * 20.0) && ply.isSteppingCarefully) {
				if (isClient) {
					// I HATE SELF-UPDATING PLAYERS
					// I HATE SELF-UPDATING PLAYERS
					// fix "bug" where shockwave doesn't trigger even when player is falling faster than orbiting satellite
					cooldown = ServerConfig.Shockwave.COOLDOWN
					MatteryPlayerNetworkChannel.sendToServer(TriggerShockwavePacket)
				} else {
					shockwave()
				}
			}

			if (!wasMidair) {
				highestSpeed = 0.0
			}
		} else if (!isActive) {
			highestSpeed = 0.0
			wasMidair = false
		}
	}

	override fun tickServer() {
		if (cooldown > 0) {
			cooldown--
		}

		ticker(false)
	}

	override fun renderIcon(stack: PoseStack, x: Float, y: Float, width: Float, height: Float) {
		if (cooldown > 0) {
			RenderSystem.setShaderColor(1f, 0.4f, 0.4f, 1f)
		}

		ResearchIcons.ICON_SHOCKWAVE.render(stack, x, y, width, height)

		if (cooldown > 0) {
			RenderSystem.setShaderColor(1f, 1f, 1f, 1f)
		}
	}

	companion object {
		val POWER_COST_DESCRIPTION =
			AndroidResearchManager.descriptionFunc(
				ResourceLocation(OverdriveThatMatters.MOD_ID, MNames.SHOCKWAVE),
				"otm.gui.power_cost_per_use",
				{ ServerConfig.Shockwave.ENERGY_COST.formatPower().copy().withStyle(ChatFormatting.YELLOW) })
	}
}
