package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.server.level.ServerLevel
import net.minecraft.util.RandomSource
import net.minecraft.world.entity.EntityType
import net.minecraft.world.entity.MobSpawnType
import net.minecraft.world.entity.SpawnGroupData
import net.minecraft.world.entity.monster.Phantom
import net.minecraft.world.level.NaturalSpawner
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.material.Material
import net.minecraft.world.level.material.MaterialColor
import net.minecraft.world.phys.AABB
import net.minecraftforge.common.ForgeHooks
import ru.dbotthepony.mc.otm.core.minus
import ru.dbotthepony.mc.otm.core.plus

class PhantomAttractorBlock : RotatableMatteryBlock(Properties.of(Material.METAL, MaterialColor.COLOR_BLUE).destroyTime(3f).explosionResistance(12f).randomTicks()) {
	@Suppress("OVERRIDE_DEPRECATION")
	override fun randomTick(
		blockState: BlockState,
		level: ServerLevel,
		blockPos: BlockPos,
		randomSource: RandomSource
	) {
		if (level.server.isSpawningMonsters && (!level.dimensionType().hasSkyLight || level.skyDarken >= 5 && level.canSeeSky(blockPos.above())) && randomSource.nextDouble() <= 0.4) {
			if (level.getEntitiesOfClass(Phantom::class.java, AABB(
				blockPos - BlockPos(20, 20, 20),
				blockPos + BlockPos(20, 20, 20),
			)).size > 4) {
				return
			}

			var groupData: SpawnGroupData? = null

			for (i in 0 .. randomSource.nextInt(2, 6)) {
				val spawnPos = blockPos + BlockPos(randomSource.nextInt(-11, 11), 20 + randomSource.nextInt(-8, 20), randomSource.nextInt(-11, 11))

				if (
					(!level.dimensionType().hasSkyLight || level.canSeeSky(spawnPos)) &&
					NaturalSpawner.isValidEmptySpawnBlock(level, spawnPos, level.getBlockState(spawnPos), level.getFluidState(blockPos), EntityType.PHANTOM)
				) {
					val phantom = EntityType.PHANTOM.create(level) ?: continue
					phantom.moveTo(spawnPos, 0.0f, 0.0f)

					if (ForgeHooks.canEntitySpawn(phantom, level, spawnPos.x.toDouble(), spawnPos.y.toDouble(), spawnPos.z.toDouble(), null, MobSpawnType.SPAWNER) == -1) {
						continue
					}

					groupData = phantom.finalizeSpawn(level, level.getCurrentDifficultyAt(spawnPos), MobSpawnType.SPAWNER, groupData, null)
					level.addFreshEntityWithPassengers(phantom)
				}
			}
		}
	}
}
