package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.ChatFormatting
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.ListTag
import net.minecraft.network.chat.Component
import net.minecraft.world.item.BlockItem
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.TooltipFlag
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.capabilities.ForgeCapabilities
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.capability.*
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.core.ifHas
import ru.dbotthepony.mc.otm.core.ifPresentK
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.map
import ru.dbotthepony.mc.otm.core.set

abstract class MatteryPoweredBlockEntity(p_155228_: BlockEntityType<*>, p_155229_: BlockPos, p_155230_: BlockState) : MatteryBlockEntity(p_155228_, p_155229_, p_155230_) {
	abstract val energy: BlockEnergyStorageImpl
	private var valid = true
	val batteryContainer = MatteryContainer(this::setChangedLight, 1)

	protected fun batteryChargeLoop() {
		var hit = false

		for (stack in batteryContainer) {
			if (!stack.isEmpty) {
				hit = true
				break
			}
		}

		if (!hit) {
			return
		}

		var demand = energy.receiveEnergyOuter(energy.missingPower, true)
		if (demand.isZero) return

		for (stack in batteryContainer) {
			if (!stack.isEmpty) {
				stack.getCapability(ForgeCapabilities.ENERGY).ifPresentK {
					if (it is IMatteryEnergyStorage) {
						val diff = it.extractEnergyOuter(demand, false)
						energy.receiveEnergyInner(diff, false)
						demand -= diff
					} else {
						val diff = it.extractEnergy(demand, false)
						energy.receiveEnergyInner(diff, false)
						demand -= diff
					}
				}

				if (demand <= ImpreciseFraction.ZERO)
					return
			}
		}
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		energy.invalidate()
		valid = false
	}

	override fun reviveCaps() {
		super.reviveCaps()
		energy.revive()
		valid = true
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid && (cap == MatteryCapability.ENERGY || cap == ForgeCapabilities.ENERGY))
			return energy.resolver.cast()

		return super.getCapability(cap, side)
	}

	public override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[ENERGY_KEY] = energy.serializeNBT()
		nbt[BATTERY_KEY] = batteryContainer.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		nbt.ifHas(ENERGY_KEY, CompoundTag::class.java, energy::deserializeNBT)
		batteryContainer.deserializeNBT(nbt[BATTERY_KEY])
		super.load(nbt)
	}

	companion object {
		fun appendHoverText(itemStack: ItemStack, blockGetter: BlockGetter?, tooltips: MutableList<Component>, flag: TooltipFlag) {
			val tag = itemStack.tag?.get(BlockItem.BLOCK_ENTITY_TAG) as? CompoundTag ?: return
			val container = MatteryContainer(1)
			tag.map(BATTERY_KEY, container::deserializeNBT)

			if (!container[0].isEmpty) {
				tooltips.add(TranslatableComponent("otm.item.block.stored_battery", container[0].displayName).withStyle(ChatFormatting.GRAY))

				ItemEnergyStorageImpl.appendHoverText(container[0], tooltips)
			}
		}
	}
}