package ru.dbotthepony.mc.otm.capability

import it.unimi.dsi.fastutil.objects.Object2IntAVLTreeMap
import net.minecraft.ChatFormatting
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.ListTag
import net.minecraft.nbt.StringTag
import net.minecraft.network.chat.Component
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import net.minecraft.tags.TagKey
import net.minecraft.world.effect.MobEffect
import net.minecraft.world.entity.Entity
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.MobSpawnType
import net.minecraft.world.entity.item.ItemEntity
import net.minecraft.world.entity.monster.Phantom
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.enchantment.EnchantmentHelper.hasVanishingCurse
import net.minecraft.world.level.GameRules
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.capabilities.ICapabilityProvider
import net.minecraftforge.common.util.INBTSerializable
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.event.AttachCapabilitiesEvent
import net.minecraftforge.event.TickEvent
import net.minecraftforge.event.TickEvent.PlayerTickEvent
import net.minecraftforge.event.entity.living.LivingDeathEvent
import net.minecraftforge.event.entity.living.LivingDropsEvent
import net.minecraftforge.event.entity.living.LivingHurtEvent
import net.minecraftforge.event.entity.living.LivingSpawnEvent
import net.minecraftforge.event.entity.living.MobEffectEvent
import net.minecraftforge.event.entity.player.EntityItemPickupEvent
import net.minecraftforge.event.entity.player.PlayerEvent
import net.minecraftforge.eventbus.api.Event
import net.minecraftforge.registries.ForgeRegistries
import org.apache.commons.lang3.mutable.MutableInt
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.mc.otm.*
import ru.dbotthepony.mc.otm.android.AndroidFeature
import ru.dbotthepony.mc.otm.android.AndroidFeatureType
import ru.dbotthepony.mc.otm.android.AndroidResearch
import ru.dbotthepony.mc.otm.android.AndroidResearchManager
import ru.dbotthepony.mc.otm.android.AndroidResearchType
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.*
import ru.dbotthepony.mc.otm.menu.ExoSuitInventoryMenu
import ru.dbotthepony.mc.otm.network.*
import ru.dbotthepony.mc.otm.registry.AndroidFeatures
import ru.dbotthepony.mc.otm.registry.MRegistry
import java.util.*
import java.util.stream.Stream
import kotlin.collections.ArrayDeque
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

@Suppress("unused")
class MatteryPlayerCapability(val ply: Player) : ICapabilityProvider, INBTSerializable<CompoundTag> {
	private inner class PlayerMatteryContainer(size: Int) : MatteryContainer(size) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)
			// TODO: Minecraft has hard-coded inventory triggers to Inventory class
			// when nothing use Inventory class directly.

			// Why?
			// CriteriaTriggers.INVENTORY_CHANGED.trigger(this, this.getInventory(), p_143468_)
		}
	}

	val synchronizer = FieldSynchronizer()

	var hasExoSuit by synchronizer.bool()

	private val exoSuitSlotCountModifiersMap: MutableMap<UUID, Int> by synchronizer.Map(
		keyCodec = UUIDValueCodec,
		valueCodec = IntValueCodec,
		backingMap = HashMap(),
		callback = {
			this.exoSuitSlotCountModifiers.recompute()
		}
	)

	val exoSuitSlotCountModifiers = UUIDIntModifiersMap(observer = observer@{
		if (ply !is ServerPlayer)
			return@observer

		if (it < 0) {
			exoSuitSlotCount = 0
		} else {
			exoSuitSlotCount = it
		}
	}, backingMap = this.exoSuitSlotCountModifiersMap)

	var exoSuitSlotCount by synchronizer.int(setter = setter@{ value, access, _ ->
		require(value >= 0) { "Invalid slot count $value" }

		if (value != access.read()) {
			access.write(value)
			_exoSuitMenu = null
			exoSuitContainer = PlayerMatteryContainer(value)
		}
	})

	var exoSuitContainer: MatteryContainer = PlayerMatteryContainer(0)
		private set(value) {
			_exoSuitMenu = null

			if (ply.containerMenu.slots.any { it.container == field }) {
				ply.closeContainer()
			}

			for (i in 0 until value.containerSize.coerceAtMost(field.containerSize)) {
				if (!field[i].isEmpty) {
					value[i] = field[i]
				}
			}

			for (i in value.containerSize until field.containerSize) {
				ply.spawnAtLocation(field[i])
			}

			field = value
		}

	fun dropExoSuitInventory(): List<ItemEntity> {
		val result = ArrayList<ItemEntity>()
		val oldCaptureDrops = ply.captureDrops(result) as MutableCollection<ItemEntity>?

		val iterator = exoSuitContainer.iterator().nonEmpty()

		for (item in iterator) {
			if (!hasVanishingCurse(item)) {
				ply.drop(item, true, false)
			}

			iterator.remove()
		}

		ply.captureDrops(oldCaptureDrops)
		oldCaptureDrops?.addAll(result)

		return result
	}

	var isExoSuitCraftingUpgraded by synchronizer.bool(setter = setter@{ value, access, _ ->
		if (value != access.read()) {
			access.write(value)
			_exoSuitMenu = null
		}
	})

	private var _exoSuitMenu: ExoSuitInventoryMenu? = null
		set(value) {
			if (field == ply.containerMenu) { ply.closeContainer() }
			field = value
		}

	val exoSuitMenu: ExoSuitInventoryMenu
		get() {
			if (_exoSuitMenu == null) {
				_exoSuitMenu = ExoSuitInventoryMenu(this)
			}

			return _exoSuitMenu!!
		}

	private var shouldSendIteration = false
	var iteration = 0
		private set

	private var lastDeathTick = 0

	private val deathLog = ArrayDeque<Pair<Int, Component>>()

	private val featureMap = IdentityHashMap<AndroidFeatureType<*>, AndroidFeature>()
	private val networkQueue = ArrayList<Any>()
	private val queuedTicks = ArrayList<Runnable>()
	private var tickedOnce = false

	private var shouldPlaySound = false
	private val research = IdentityHashMap<AndroidResearchType, AndroidResearch>()

	private var invalidateNetworkIn = 10

	val isEverAndroid: Boolean get() = isAndroid || willBecomeAndroid
	var lastJumpTicks = 14

	var ticksIExist = 0
		private set

	var willBecomeAndroid by synchronizer.bool()
	var isAndroid by synchronizer.bool()

	val androidEnergy = SynchronizedPowerWithBattery(ply, synchronizer, ServerConfig.ANDROID_MAX_ENERGY, ServerConfig.ANDROID_MAX_ENERGY)

	fun invalidateNetworkState() {
		invalidateNetworkIn = 10
	}

	fun becomeAndroidSoft() {
		if (isAndroid || willBecomeAndroid) return
		willBecomeAndroid = true
		(ply as? ServerPlayer)?.displayClientMessage(TranslatableComponent("otm.pill.message").withStyle(ChatFormatting.GRAY), false)
	}

	fun becomeAndroid() {
		if (isAndroid) return

		isAndroid = true
		willBecomeAndroid = false
		shouldPlaySound = false
		iteration = 0
		deathLog.clear()
		androidEnergy.batteryLevel = ServerConfig.ANDROID_MAX_ENERGY
		androidEnergy.maxBatteryLevel = ServerConfig.ANDROID_MAX_ENERGY
	}

	fun becomeAndroidAndKill() {
		if (isAndroid) return

		becomeAndroid()
		ply.hurt(MRegistry.DAMAGE_BECOME_ANDROID, ply.maxHealth * 2)
	}

	fun becomeHumane() {
		if (willBecomeAndroid) willBecomeAndroid = false
		if (!isAndroid) return

		isAndroid = false
		shouldPlaySound = false
		iteration = 0
		deathLog.clear()
		androidEnergy.batteryLevel = ImpreciseFraction.ZERO
		androidEnergy.maxBatteryLevel = ServerConfig.ANDROID_MAX_ENERGY
		dropBattery()
	}

	fun becomeHumaneAndKill() {
		if (willBecomeAndroid) willBecomeAndroid = false
		if (!isAndroid) return

		becomeHumane()
		ply.hurt(MRegistry.DAMAGE_BECOME_HUMANE, ply.maxHealth * 2)
	}

	fun obliviate(refund: Boolean = true) {
		for (instance in research.values) {
			if (instance.isResearched) {
				instance.unResearch()

				if (refund) {
					instance.refund(simulate = false)
				}
			}
		}

		val copy = ArrayList<AndroidFeature>(featureMap.values.size).also { it.addAll(featureMap.values) }

		for (feature in copy) {
			removeFeature(feature.type)
		}

		iteration = 0
		deathLog.clear()
	}

	fun getResearch(type: AndroidResearchType): AndroidResearch {
		return research.computeIfAbsent(type) {
			return@computeIfAbsent AndroidResearch(type, this)
		}
	}

	fun reloadResearch() {
		val old = ArrayList<AndroidResearch>(research.size)
		old.addAll(research.values)
		research.clear()

		for (v in old) {
			val new = AndroidResearchManager.get(v.type.id) ?: continue

			research[new] = AndroidResearch(new, this).also {
				it.deserializeNBT(v.serializeNBT())
			}
		}
	}

	val features: Stream<out AndroidFeature> get() = featureMap.values.stream()

	private fun addFeature(feature: AndroidFeature): Boolean {
		if (featureMap.containsKey(feature.type)) return false
		featureMap[feature.type] = feature

		if (!ply.level.isClientSide) {
			queuedTicks.add(feature::applyModifiers)
		}

		if (ply is ServerPlayer) {
			feature.invalidateNetwork()
			val payload = feature.collectNetworkPayload()

			if (payload != null) {
				sendNetwork(AndroidFeatureSyncPacket(feature.type, payload, null))
			}
		}

		return true
	}

	@Suppress("unchecked_cast")
	fun <T : AndroidFeature> addFeature(feature: AndroidFeatureType<T>): T {
		val get = featureMap[feature]
		if (get != null) return get as T

		val factory = feature.create(this)

		featureMap[feature] = factory

		if (!ply.level.isClientSide) {
			queuedTicks.add(factory::applyModifiers)
		}

		if (ply is ServerPlayer) {
			factory.invalidateNetwork()
			val payload = factory.collectNetworkPayload()

			if (payload != null) {
				sendNetwork(AndroidFeatureSyncPacket(feature, payload, null))
			}
		}

		return factory
	}

	fun removeFeature(feature: AndroidFeatureType<*>): Boolean {
		val removed = featureMap.remove(feature)

		if (removed != null) {
			if (!ply.level.isClientSide) {
				queuedTicks.add {
					removed.removeModifiers()
				}
			}

			if (ply is ServerPlayer) {
				sendNetwork(AndroidFeatureRemovePacket(removed.type))
			}
		}

		return removed != null
	}

	fun hasFeature(feature: AndroidFeatureType<*>): Boolean {
		return featureMap.containsKey(feature)
	}

	fun hasFeatureLevel(feature: AndroidFeatureType<*>, level: Int): Boolean {
		val get = featureMap[feature] ?: return false
		return get.level >= level
	}

	@Suppress("unchecked_cast")
	fun <T : AndroidFeature> getFeature(feature: AndroidFeatureType<T>): T? {
		return featureMap[feature] as T?
	}

	fun onHurt(event: LivingHurtEvent) {
		for (feature in featureMap.values) {
			feature.onHurt(event)
		}
	}

	fun <T : AndroidFeature> computeIfAbsent(feature: AndroidFeatureType<T>): T {
		return getFeature(feature) ?: addFeature(feature)
	}

	fun <T : AndroidFeature> getFeatureO(feature: AndroidFeatureType<T>): Optional<T> {
		val get = getFeature(feature)
		return if (get != null) Optional.of(get) else Optional.empty()
	}

	fun <T : AndroidFeature> ifFeature(feature: AndroidFeatureType<T>, consumer: (T) -> Unit) {
		val get = getFeature(feature)

		if (get != null) {
			consumer(get)
		}
	}

	override fun serializeNBT(): CompoundTag {
		val tag = CompoundTag()

		tag["ticksIExist"] = ticksIExist

		// iteration
		tag["iteration"] = iteration
		tag["shouldSendIteration"] = shouldSendIteration

		tag["deathLog"] = ListTag().also {
			for ((ticks, component) in deathLog) {
				it.add(CompoundTag().also {
					it["ticks"] = ticks
					it["component"] = StringTag.valueOf(Component.Serializer.toJson(component))
				})
			}
		}

		// exosuit
		tag["hasExoSuit"] = hasExoSuit
		tag["exoSuitContainer"] = exoSuitContainer.serializeNBT()
		tag["isExoSuitCraftingUpgraded"] = isExoSuitCraftingUpgraded
		tag["exoSuitSlotCountModifiers"] = exoSuitSlotCountModifiers.serializeNBT()

		// android
		tag["androidEnergy"] = androidEnergy.serializeNBT()

		tag["isAndroid"] = isAndroid
		tag["willBecomeAndroid"] = willBecomeAndroid

		val featureList = ListTag()
		val researchList = ListTag()

		for (feature in featureMap.values) {
			featureList.add(feature.serializeNBT().also {
				it["id"] = feature.type.registryName!!.toString()
			})
		}

		for ((type, instance) in research) {
			researchList.add(instance.serializeNBT().also {
				it["id"] = type.id.toString()
			})
		}

		tag["features"] = featureList
		tag["research"] = researchList

		return tag
	}

	override fun deserializeNBT(tag: CompoundTag) {
		ticksIExist = tag.getInt("ticksIExist")

		// iterations
		iteration = tag.getInt("iteration")
		shouldSendIteration = tag.getBoolean("shouldSendIteration")

		deathLog.clear()

		for (value in tag.getCompoundList("deathLog")) {
			val component = Component.Serializer.fromJson(value.getString("component"))

			if (component != null) {
				deathLog.add(value.getInt("ticks") to component)
			}
		}

		// exosuit
		hasExoSuit = tag.getBoolean("hasExoSuit")
		tag.map("exoSuitSlotCountModifiers", exoSuitSlotCountModifiers::deserializeNBT)

		exoSuitContainer.deserializeNBT(tag["exoSuitContainer"])
		isExoSuitCraftingUpgraded = tag.getBoolean("isExoSuitCraftingUpgraded")

		// android
		isAndroid = tag.getBoolean("isAndroid")
		willBecomeAndroid = tag.getBoolean("willBecomeAndroid")

		tag.map("androidEnergy", androidEnergy::deserializeNBT)

		featureMap.clear()
		research.clear()

		for (featureTag in tag.getCompoundList("features")) {
			val feature = MRegistry.ANDROID_FEATURES.getValue(ResourceLocation(featureTag.getString("id")))

			if (feature?.isApplicable(this) == true) {
				val instance = feature.create(this)

				instance.deserializeNBT(featureTag)
				addFeature(instance)

				if (!ply.level.isClientSide) {
					queuedTicks.add(instance::applyModifiers)
				}
			}
		}

		for (researchTag in tag.getCompoundList("research")) {
			val research = AndroidResearchManager[ResourceLocation(researchTag.getString("id"))]

			if (research != null) {
				val instance = AndroidResearch(research, this)
				instance.deserializeNBT(researchTag)
				this.research[research] = instance
			}
		}
	}

	fun dropBattery() {
		if (androidEnergy.item.isEmpty) return
		ply.spawnAtLocation(androidEnergy.item)
		androidEnergy.item = ItemStack.EMPTY
	}

	private fun sendNetwork(packet: Any) {
		if (ply is ServerPlayer) {
			if (tickedOnce) {
				MatteryPlayerNetworkChannel.send(ply, packet)
			} else {
				networkQueue.add(packet)
			}
		}
	}

	private fun tickInventory() {
		if (hasExoSuit) {
			for ((i, stack) in exoSuitContainer.withIndex()) {
				if (!stack.isEmpty) {
					stack.inventoryTick(ply.level, ply, i + 41, false)
				}
			}
		}
	}

	fun tickClient() {
		queuedTicks.clear()

		if (!ply.isAlive) {
			return
		}

		if (isAndroid) {
			for (feature in featureMap.values) {
				feature.tickClient()
			}
		}

		tickInventory()
	}

	fun preTick() {
		if (!ply.isAlive) return

		if (isAndroid) {
			ForgeRegistries.MOB_EFFECTS.tags()?.getTag(ANDROID_IMMUNE_EFFECTS)?.forEach {
				if (ply.hasEffect(it))
					ply.removeEffect(it)
			}
		}
	}

	fun tick() {
		if (!ply.isAlive) return

		ticksIExist++

		if (willBecomeAndroid) {
			if (ply.isSleeping && ply.sleepTimer > SLEEP_TICKS_LIMIT) {
				becomeAndroid()
				shouldPlaySound = true
				(ply as? ServerPlayer)?.displayClientMessage(TranslatableComponent("otm.pill.message_finish").withStyle(ChatFormatting.DARK_RED), false)
			}
		}

		if (isAndroid) {
			if (ply.airSupply < ply.maxAirSupply)
				ply.airSupply = ply.maxAirSupply

			// TODO: Maybe passive drain?
			// extractEnergyInner(BigDecimal.valueOf(new Random().nextDouble()), false);
			if (ply.isSwimming && !hasFeature(AndroidFeatures.AIR_BAGS)) {
				ply.isSwimming = false
			}

			androidEnergy.tick()

			val stats = ply.foodData

			while (stats.foodLevel < 18 && androidEnergy.extractEnergyInner(ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT, true) >= ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT) {
				androidEnergy.extractEnergyInner(ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT, false)
				stats.foodLevel = stats.foodLevel + 1
			}

			// "block" quick regeneration
			// also cause power to generate while in peaceful
			while (stats.foodLevel > 18 && androidEnergy.receiveEnergyInner(ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT, true) >= ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT) {
				androidEnergy.receiveEnergyInner(ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT, false)
				stats.foodLevel = stats.foodLevel - 1
			}

			val foodLevel = stats.foodLevel.toFloat()

			if (stats.saturationLevel < foodLevel) {
				val extracted = androidEnergy.extractEnergyInner(ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT * (foodLevel - stats.saturationLevel), false)
				stats.setSaturation(stats.saturationLevel + (extracted / ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT).toFloat())
			}

			if (stats.exhaustionLevel > 0f) {
				val extracted = androidEnergy.extractEnergyInner(ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT * (stats.exhaustionLevel / 4f), false)
				stats.setExhaustion(stats.exhaustionLevel - (extracted / ServerConfig.ANDROID_ENERGY_PER_HUNGER_POINT).toFloat() * 4f)
			}

			for (feature in featureMap.values) {
				feature.tickServer()
			}
		}

		for (runnable in queuedTicks) {
			runnable.run()
		}

		queuedTicks.clear()

		if (invalidateNetworkIn > 0 && --invalidateNetworkIn == 0) {
			synchronizer.invalidate()

			for (instance in research.values) {
				instance.invalidateNetwork()
			}

			for (feature in featureMap.values) {
				feature.invalidateNetwork()
			}
		}

		tickedOnce = true

		val payload = synchronizer.collectNetworkPayload()

		if (payload != null) {
			MatteryPlayerNetworkChannel.send(ply, MatteryPlayerFieldPacket(payload))
		}

		if (networkQueue.size != 0) {
			for (packet in networkQueue) {
				MatteryPlayerNetworkChannel.send(ply, packet)
			}

			networkQueue.clear()
		}

		for (instance in research.values) {
			val researchPayload = instance.collectNetworkPayload()

			if (researchPayload != null) {
				sendNetwork(AndroidResearchSyncPacket(instance.type, researchPayload, null))
			}
		}

		for (instance in featureMap.values) {
			val featurePayload = instance.collectNetworkPayload()

			if (featurePayload != null) {
				sendNetwork(AndroidFeatureSyncPacket(instance.type, featurePayload, null))
			}
		}

		if (shouldSendIteration) {
			sendNetwork(PlayerIterationPacket(iteration, deathLog))
			shouldSendIteration = false
		}

		if (hasExoSuit && ply.containerMenu == ply.inventoryMenu) {
			exoSuitMenu.broadcastChanges()
		}

		tickInventory()
	}

	private val resolver = LazyOptional.of { this }

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		return if (cap == MatteryCapability.MATTERY_PLAYER) {
			resolver.cast()
		} else LazyOptional.empty()
	}

	@Suppress("unused")
	companion object {
		val ANDROID_IMMUNE_EFFECTS: TagKey<MobEffect> = TagKey.create(ForgeRegistries.MOB_EFFECTS.registryKey, ResourceLocation(OverdriveThatMatters.MOD_ID, "android_immune_effects"))

		fun onPlayerTick(event: PlayerTickEvent) {
			val ent = event.player

			if (event.phase == TickEvent.Phase.START) {
				if (!ent.level.isClientSide) {
					ent.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK {
						it.preTick()
					}
				}
			} else {
				if (ent.level.isClientSide) {
					ent.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK {
						it.tickClient()
					}
				} else {
					ent.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK {
						it.tick()
					}
				}
			}
		}

		fun isMobEffectApplicable(event: MobEffectEvent.Applicable) {
			event.entity.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK {
				if (it.isAndroid && ForgeRegistries.MOB_EFFECTS.tags()?.getTag(ANDROID_IMMUNE_EFFECTS)?.stream()?.anyMatch { it == event.effectInstance.effect } == true) {
					event.result = Event.Result.DENY
				}
			}
		}

		fun onHurtEvent(event: LivingHurtEvent) {
			if (event.isCanceled) {
				return
			}

			event.entity.getCapability(MatteryCapability.MATTERY_PLAYER).ifPresentK { it.onHurt(event) }
		}

		const val CAPABILITY_KEY = "otm_player"
		val CAPABILITY_LOCATION = ResourceLocation(OverdriveThatMatters.MOD_ID, CAPABILITY_KEY)

		fun onAttachCapabilityEvent(event: AttachCapabilitiesEvent<Entity>) {
			val ent = event.`object`

			if (ent is Player) {
				event.addCapability(CAPABILITY_LOCATION, MatteryPlayerCapability(ent))
			}
		}

		fun onPlayerChangeDimensionEvent(event: PlayerEvent.PlayerChangedDimensionEvent) {
			event.entity.getCapability(MatteryCapability.MATTERY_PLAYER)
				.ifPresentK { it.invalidateNetworkState() }
		}

		private val waitingToDie = ArrayList<Pair<LivingEntity, MatteryPlayerCapability>>()

		fun onLivingDrops(event: LivingDropsEvent) {
			val indexOf = waitingToDie.indexOfFirst { event.entity == it.first }

			if (indexOf == -1)
				return

			val drops = waitingToDie[indexOf].second.dropExoSuitInventory()

			if (event.entity.captureDrops() == null) {
				event.drops.addAll(drops)
			}

			waitingToDie.removeAt(indexOf)
		}

		fun onPlayerDeath(event: LivingDeathEvent) {
			val ply = event.entity as? Player ?: return
			val mattery = ply.matteryPlayer ?: return

			if (mattery.lastDeathTick != ply.tickCount) {
				mattery.lastDeathTick = ply.tickCount
			} else {
				return
			}

			if (mattery.hasExoSuit && !ply.level.gameRules.getBoolean(GameRules.RULE_KEEPINVENTORY)) {
				waitingToDie.add(ply to mattery)

				onceServer {
					val index = waitingToDie.indexOfFirst { it.first == ply }

					if (index != -1) {
						waitingToDie.removeAt(index)
						mattery.dropExoSuitInventory().forEach { ply.level.addFreshEntity(it) }
						LOGGER.warn("Unable to 'properly' drop $ply's ExoSuit inventory contents")
						LOGGER.warn("I blame Forge.")
					}
				}
			}

			if (!mattery.isAndroid) {
				return
			}

			mattery.iteration++
			mattery.shouldSendIteration = true
			mattery.deathLog.addLast(ply.tickCount to ply.combatTracker.deathMessage)

			while (mattery.deathLog.size > 6) {
				mattery.deathLog.removeFirst()
			}
		}

		fun onPlayerCloneEvent(event: PlayerEvent.Clone) {
			val it = event.entity.matteryPlayer ?: return

			var resolver = event.original.getCapability(MatteryCapability.MATTERY_PLAYER)

			if (!resolver.isPresent || resolver.resolve().isEmpty) {
				event.original.reviveCaps()
				resolver = event.original.getCapability(MatteryCapability.MATTERY_PLAYER)
			}

			if (!resolver.isPresent || resolver.resolve().isEmpty) {
				event.original.invalidateCaps()
				return
			}

			val original = resolver.resolve().get()

			if (original.willBecomeAndroid && event.isWasDeath) {
				original.becomeAndroid()
				(event.entity as? ServerPlayer)?.displayClientMessage(TranslatableComponent("otm.pill.message_finish").withStyle(ChatFormatting.DARK_RED), false)
			}

			it.deserializeNBT(original.serializeNBT())
			it.invalidateNetworkState()
			event.original.invalidateCaps()
		}

		const val SLEEP_TICKS_LIMIT = 80

		private val itemPickupTicks = WeakHashMap<Player, WeakHashMap<ItemEntity, MutableInt>>()

		private val LOGGER = LogManager.getLogger()

		fun onPickupEvent(event: EntityItemPickupEvent) {
			if (event.item.owner != null && event.item.owner != event.entity.uuid && event.item.age < 200 || event.item.item.isEmpty) {
				return
			}

			val it = event.entity.matteryPlayer ?: return

			if (!it.hasExoSuit) {
				return
			}

			val ticks = itemPickupTicks.computeIfAbsent(event.entity) { WeakHashMap() }.computeIfAbsent(event.item) { MutableInt() }
			ticks.increment()

			if (ticks.value < 10) {
				return
			}

			val leftover = it.exoSuitContainer.addItem(event.item.item, false)
			event.item.item.count = leftover.count
		}

		fun canEntitySpawn(event: LivingSpawnEvent.CheckSpawn) {
			if (event.spawnReason == MobSpawnType.NATURAL && event.entity is Phantom) {
				for (ply in event.entity.level.players()) {
					if (ply.matteryPlayer?.isAndroid == true) {
						if (
							ply.position.y in event.y - 36.0 .. event.y - 19.0 &&
							ply.position.x in event.x - 12.0 .. event.x + 12.0 &&
							ply.position.z in event.z - 12.0 .. event.z + 12.0
						) {
							event.result = Event.Result.DENY
							return
						}
					}
				}
			}
		}
	}
}
