package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.screens.Screen
import net.minecraft.network.chat.Component
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.client.render.SkinElement
import ru.dbotthepony.mc.otm.client.render.WidgetLocation
import ru.dbotthepony.mc.otm.client.screen.panels.CheckBoxPanel.Companion.REGULAR_DIMENSIONS
import ru.dbotthepony.mc.otm.menu.widget.BooleanPlayerInputWidget

open class CheckBoxPanel<out S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = REGULAR_DIMENSIONS,
	height: Float = REGULAR_DIMENSIONS
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	open var checked = false
	open var isDisabled = false

	override fun innerRender(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		if (checked) {
			CHECKBOX_CHECKED.render(stack)
		} else {
			CHECKBOX_UNCHECKED.render(stack)
		}
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (!isDisabled) {
			checked = !checked
			playGuiClickSound()
		}

		return true
	}

	companion object {
		const val REGULAR_DIMENSIONS = 15f

		val CHECKBOX_UNCHECKED = SkinElement(
			WidgetLocation.WIDGETS,
			x = 59f,
			y = 241f,
			width = REGULAR_DIMENSIONS,
			height = REGULAR_DIMENSIONS,
		)

		val CHECKBOX_CHECKED = SkinElement(
			WidgetLocation.WIDGETS,
			x = 44f,
			y = 241f,
			width = REGULAR_DIMENSIONS,
			height = REGULAR_DIMENSIONS,
		)
	}
}

open class CheckBoxInputPanel<out S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	val widget: BooleanPlayerInputWidget,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = REGULAR_DIMENSIONS + 120f,
	height: Float = REGULAR_DIMENSIONS
) : CheckBoxPanel<S>(screen, parent, x, y, width, height) {
	override var checked: Boolean
		get() = widget.value
		set(value) {}

	override var isDisabled: Boolean
		get() = !widget.ignoreSpectators || minecraft.player?.isSpectator == true
		set(value) {}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		super.mouseClickedInner(x, y, button)

		if (!isDisabled) {
			widget.userInput(!checked, minecraft.player)
		}

		return true
	}
}

open class CheckBoxLabelPanel<out S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	text: Component,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = REGULAR_DIMENSIONS + 120f,
	height: Float = REGULAR_DIMENSIONS
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	val checkbox = CheckBoxPanel(screen, this, 0f, 0f, REGULAR_DIMENSIONS, REGULAR_DIMENSIONS)
	val label = Label(screen, this, REGULAR_DIMENSIONS + 4f, 4f, text = text)
}

open class CheckBoxLabelInputPanel<out S : Screen> @JvmOverloads constructor(
	screen: S,
	parent: EditablePanel<*>?,
	widget: BooleanPlayerInputWidget,
	text: Component,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = REGULAR_DIMENSIONS + 120f,
	height: Float = REGULAR_DIMENSIONS
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	val widget get() = checkbox.widget
	val checkbox = CheckBoxInputPanel(screen, this, widget, 0f, 0f, REGULAR_DIMENSIONS, REGULAR_DIMENSIONS)
	val label = Label(screen, this, REGULAR_DIMENSIONS + 4f, 4f, text = text)
}
