package ru.dbotthepony.mc.otm.menu.widget

import net.minecraft.network.FriendlyByteBuf
import net.minecraft.world.entity.player.Player
import net.minecraftforge.network.NetworkEvent
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.network.MatteryPacket
import ru.dbotthepony.mc.otm.network.MenuNetworkChannel
import ru.dbotthepony.mc.otm.network.sender
import java.util.function.Supplier
import kotlin.reflect.KMutableProperty0

class BooleanPlayerInputPacket(val id: Int, val value: Boolean) : MatteryPacket {
	override fun play(context: Supplier<NetworkEvent.Context>) {
		context.get().packetHandled = true

		context.get().enqueueWork {
			((context.get().sender?.containerMenu as? MatteryMenu)?.getWidget(id) as? BooleanPlayerInputWidget)?.userInput(value, context.sender!!)
		}
	}

	override fun write(buff: FriendlyByteBuf) {
		buff.writeInt(id)
		buff.writeBoolean(value)
	}

	companion object {
		fun read(buff: FriendlyByteBuf): BooleanPlayerInputPacket {
			return BooleanPlayerInputPacket(buff.readInt(), buff.readBoolean())
		}
	}
}

class BooleanPlayerInputWidget(menu: MatteryMenu) : AbstractWidget(menu) {
	var value by menu.mSynchronizer.bool()
	var ignoreSpectators by menu.mSynchronizer.bool(true)

	constructor(menu: MatteryMenu, state: KMutableProperty0<Boolean>) : this(menu) {
		withClicker { state.set(it) }
		withSupplier { state.get() }
	}

	var supplier: (() -> Boolean)? = null
	var clicker: ((Boolean) -> Unit)? = null

	fun withSupplier(func: () -> Boolean): BooleanPlayerInputWidget {
		supplier = func
		return this
	}

	fun withClicker(func: (Boolean) -> Unit): BooleanPlayerInputWidget {
		clicker = func
		return this
	}

	fun withProperty(state: KMutableProperty0<Boolean>): BooleanPlayerInputWidget {
		withClicker { state.set(it) }
		withSupplier { state.get() }
		return this
	}

	fun withoutAnything(): BooleanPlayerInputWidget {
		supplier = null
		clicker = null
		return this
	}

	fun asClient(): BooleanPlayerInputWidget {
		supplier = null
		clicker = {
			MenuNetworkChannel.sendToServer(BooleanPlayerInputPacket(slotID, it))
		}

		return this
	}

	fun userInput(newValue: Boolean, ply: Player? = null) {
		if (ply?.isSpectator == true && ignoreSpectators) {
			return
		}

		val clicker = clicker

		if (clicker != null) {
			clicker.invoke(newValue)
			updateServer()
		}
	}

	fun switchValue(ply: Player? = null) {
		userInput(!value, ply)
	}

	override fun updateServer() {
		val supplier = supplier

		if (supplier != null) {
			value = supplier.invoke()
		}
	}
}
